/*
 * File:     RemoteDCSelectPanel.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.dcr;

import mpi.dcr.ISO12620.Profile;
import mpi.dcr.ISO12620.ProfileList;

import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionListener;


/**
 * A panel with 2 or 3 sub panels, one with a list of (pre) selected profiles,
 * one with the data categories of the selected profile and a description
 * panel for the selected data category.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class RemoteDCSelectPanel extends AbstractDCSelectPanel
    implements ActionListener, ItemListener, ListSelectionListener,
        TreeSelectionListener {
    /** a constant for all profiles */
    public static final int ALL_PROFILES = 0;

    /** a constant for a selection of profiles */
    public static final int PROFILE_SELECTION = 1;

    /** the add profiles button */
    protected JButton addProfButton;

    /** the remove profiles button */
    protected JButton removeProfButton;
    private int mode = ALL_PROFILES;

    //protected JButton refreshButton;

    /**
     * Creates a new RemoteDCSelectPanel instance
     *
     * @param connector the connector
     */
    public RemoteDCSelectPanel(ILATDCRConnector connector) {
        super(connector);

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        initComponents();
    }

    /**
     * Creates a new RemoteDCSelectPanel instance
     *
     * @param connector the connector
     * @param resBundle a resource bundle
     */
    public RemoteDCSelectPanel(ILATDCRConnector connector,
        ResourceBundle resBundle) {
        super(connector, resBundle);

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        initComponents();
    }

    /**
     * Creates a new RemoteDCSelectPanel instance
     *
     * @param connector the connector
     * @param mode all profiles or profile selection mode
     */
    public RemoteDCSelectPanel(ILATDCRConnector connector, int mode) {
        super(connector);

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        if (mode == PROFILE_SELECTION) {
            this.mode = PROFILE_SELECTION; //otherwise all profiles 
        }

        initComponents();
    }

    /**
     * Creates a new RemoteDCSelectPanel instance
     *
     * @param connector the connector
     * @param resBundle the resource bundle
     * @param mode all profiles or profile selection mode
     */
    public RemoteDCSelectPanel(ILATDCRConnector connector,
        ResourceBundle resBundle, int mode) {
        super(connector, resBundle);

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        if (mode == PROFILE_SELECTION) {
            this.mode = PROFILE_SELECTION; //otherwise all profiles 
        }

        initComponents();
    }

    /**
     * Creates a new RemoteDCSelectPanel instance
     */
    public RemoteDCSelectPanel() {
        super();
        initComponents();
    }

    /**
     * Initializes the ui components.
     */
    protected void initComponents() {
        super.initComponents();

        // profile panel
        if (mode == PROFILE_SELECTION) {
            addProfButton = new JButton();
            removeProfButton = new JButton();

            Insets insets = new Insets(2, 6, 2, 6);

            addProfButton.addActionListener(this);
            removeProfButton.addActionListener(this);

            JPanel buttonPanel = new JPanel(new GridLayout(2, 1, 6, 2));
            buttonPanel.add(addProfButton);
            buttonPanel.add(removeProfButton);

            GridBagConstraints gbc = new GridBagConstraints();
            gbc.anchor = GridBagConstraints.NORTHWEST;
            gbc.insets = insets;
            gbc.gridy = 1;
            gbc.fill = GridBagConstraints.NONE;
            gbc.weightx = 0.0;
            gbc.weighty = 0.0;
            profPanel.add(buttonPanel, gbc);
        } else {
            // load all profiles
            if (connector != null) {
                try {
                    ProfileList profileList = connector.getProfiles();
                    Profile[] profs = profileList.getProfiles();
                    List ap = new ArrayList(16);

                    for (int i = 0; i < profs.length; i++) {
                        ap.add(profs[i].getContentAsString());
                    }

                    addProfiles(ap);
                } catch (DCRConnectorException dce) {
                    // log
                    dce.printStackTrace();
                }
            }
        }

        updateLocale();
    }

    /**
     * Applies localized texts to the elements, if provided.
     */
    protected void updateLocale() {
        super.updateLocale();

        String addProf = "Add Profile";
        String remProf = "Remove Profile";
        String selCats = "Select Categories";

        if (resBundle != null) {
            try {
                addProf = resBundle.getString("DCR.Label.AddProfile");
            } catch (Exception ex) {
            }

            try {
                remProf = resBundle.getString("DCR.Label.RemoveProfile");
            } catch (Exception ex) {
            }

            try {
                selCats = resBundle.getString("DCR.Label.SelectCategories");
            } catch (Exception ex) {
            }
        }

        if (mode == PROFILE_SELECTION) {
            addProfButton.setText(addProf);
            removeProfButton.setText(remProf);
        }

        catPanel.setBorder(new TitledBorder(selCats));

        /*
           sortModeCombo.removeItemListener(this);
           sortModeCombo.removeAllItems();
           for (int i = 0; i < sortModes.length; i++) {
               sortModeCombo.addItem(sortModes[i]);
           }
         */
        sortModeCombo.addItemListener(this);
    }

    /**
     * Adds the profiles in the specified list to the current list.
     *
     * @param profiles profiles to add
     */
    public void addProfiles(List profiles) {
        if (profiles != null) {
            String prof;
profilesloop: 
            for (int i = 0; i < profiles.size(); i++) {
                prof = (String) profiles.get(i);

                if (prof == null) {
                    continue;
                }

                for (int j = 0; j < profList.getModel().getSize(); j++) {
                    if (prof.equals(profList.getModel().getElementAt(j))) {
                        continue profilesloop;
                    }
                }

                // not in the list already
                ((DefaultListModel) profList.getModel()).addElement(prof);
            }

            if (profList.getModel().getSize() == 1) {
                profList.setSelectedIndex(0);
            }
        }
    }

    private void addProfiles() {
        ProfileSelectPanel pp = new ProfileSelectPanel(connector);

        int ret = JOptionPane.showConfirmDialog(this, pp, "",
                JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);

        if (ret == JOptionPane.OK_OPTION) {
            addProfiles(pp.getSelectedProfileNames());
        }
    }

    private void removeProfile() {
        if (profList.getSelectedIndex() > -1) {
            profModel.remove(profList.getSelectedIndex());
        }
    }

    /**
     * Returns a list of selected profiles.
     *
     * @return a list of selected profiles
     */
    public List getSelectedProfiles() {
        List selProfs = new ArrayList();

        for (int j = 0; j < profList.getModel().getSize(); j++) {
            selProfs.add(profList.getModel().getElementAt(j));
        }

        return selProfs;
    }

    /**
     * @see org.mpi.DCR.AbstractDCSelectPanel#getSelectedCategories()
     */
    public List getSelectedCategories() {
        List selList = super.getSelectedCategories();

        // check if all DCSmall objects are fully loaded
        DCSmall dcSmall;

        for (int i = 0; i < selList.size(); i++) {
            dcSmall = (DCSmall) selList.get(i);

            if ((dcSmall != null) && !dcSmall.isLoaded()) {
                if (connector instanceof LATDCRConnector) {
                    ((LATDCRConnector) connector).getMoreInfo(dcSmall);
                }
            }
        }

        return selList;
    }

    /**
     * Returns a list of data categories for the given profile.
     *
     * @param profile the name of the profile
     *
     * @return a list of data categories
     */
    protected List getDataCategories(String profile) {
        List datCats = null;

        try {
            datCats = connector.getDCSmallList(profile, null);

            if (sortMode == BY_ALPHA) {
                Collections.sort(datCats, new DCIdentifierComparator());
            } else if (sortMode == BY_ID) {
                Collections.sort(datCats, new DCIdComparator());
            }
        } catch (DCRConnectorException dce) {
            // show message
        } catch (IllegalArgumentException iae) {
            // show message
        }

        currentProfile = profile;

        return datCats;
    }

    /**
     * If the sorting is by broader concept, first check if all objects are
     * fully  loaded.
     *
     * @see org.mpi.DCR.AbstractDCSelectPanel#updateCategories(java.util.List)
     */
    protected void updateCategories(List datcats) {
        if (datcats == null) {
            //return; // or clear the list??
            datcats = new ArrayList(0);
        }

        if (sortMode == BY_CONCEPT) {
            DCSmall nextDC = null;

            for (int i = 0; i < datcats.size(); i++) {
                nextDC = (DCSmall) datcats.get(i);

                if (!nextDC.isLoaded()) {
                    if (connector instanceof LATDCRConnector) {
                        ((LATDCRConnector) connector).getMoreInfo(nextDC);
                    }
                }
            }
        }

        super.updateCategories(datcats);
    }

    /**
     * First checks whether the object is fully loaded, then calls super
     * implmentation.
     *
     * @see org.mpi.DCR.AbstractDCSelectPanel#updateDescription(org.mpi.DCR.DCSmall)
     */
    protected void updateDescription(DCSmall dc) {
        if ((dc != null) && !dc.isLoaded()) {
            if (connector instanceof LATDCRConnector) {
                ((LATDCRConnector) connector).getMoreInfo(dc);
            }
        }

        super.updateDescription(dc);
    }

    /**
     * The action event handling
     *
     * @param e the action event
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == addProfButton) {
            addProfiles();
        } else if (e.getSource() == removeProfButton) {
            removeProfile();
        }
    }
}
