/*
 * File:     ELAN.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator;

import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.client.annotator.util.FileUtility;

import java.awt.EventQueue;

import java.io.File;


/**
 * The main class for ELAN. Main performs some initialization and creates the
 * first frame. Holds version information, major, minor and micro.
 */
public class ELAN implements ClientLogger {
    /** the major version value */
    public static int major = 3;

    /** the minor version value */
    public static int minor = 7;

    /** the micro (bug fix) version value */
    public static int micro = 1;

    /**
     * Creates a new ELAN instance
     */
    private ELAN() {
    }

    /**
     * Main method, initialization and first frame.
     *
     * @param args the arguments, path to an eaf file
     */
    public static void main(final String[] args) {
        LOG.info("ELAN " + getVersionString());
        LOG.info("Java version: " + System.getProperty("java.version"));
        LOG.info("Runtime version: " +
            System.getProperty("java.runtime.version"));
        LOG.info("OS name: " + System.getProperty("os.name"));
        LOG.info("OS version: " + System.getProperty("os.version"));
        LOG.info("User language: " + System.getProperty("user.language"));
        LOG.info("Classpath: " + System.getProperty("java.class.path"));

        // webstart initialisation
        mpi.eudico.client.annotator.util.SystemInstallationSecurity.Instance()
                                                                   .go();

        // make sure the directory for Elan data exists, could move to preferences?
        try {
            /* HS May 2008: copy files to the new ELAN data folder. Do this only once. */
            if (System.getProperty("os.name").indexOf("Mac OS") > -1) {
                File dataFolder = new File(Constants.ELAN_DATA_DIR);

                if (!dataFolder.exists()) {
                    dataFolder.mkdir();

                    File oldDataFolder = new File(Constants.USERHOME +
                            Constants.FILESEPARATOR + ".elan_data");

                    if (oldDataFolder.exists()) {
                        // copy files
                        File[] files = oldDataFolder.listFiles();
                        File inFile = null;
                        File outFile = null;

                        for (int i = 0; i < files.length; i++) {
                            inFile = files[i];

                            if (inFile.isFile()) {
                                outFile = new File(dataFolder.getAbsolutePath() +
                                        Constants.FILESEPARATOR +
                                        inFile.getName());
                                FileUtility.copyToFile(inFile, outFile);
                            }
                        }
                    }
                }
            }

            File dataDir = new File(Constants.ELAN_DATA_DIR);

            if (!dataDir.exists()) {
                dataDir.mkdir();
            }

            // temporary, clean up old crap
            File oldCrap = new File(Constants.STRPROPERTIESFILE);
            oldCrap.delete();
            oldCrap = new File(Constants.USERHOME + Constants.FILESEPARATOR +
                    ".elan.pfs");
            oldCrap.delete();
        } catch (Exception ex) {
            // catch any
            LOG.warning("Could not create ELAN's data directory: " +
                ex.getMessage());
        }

        FrameManager.getInstance().setExitAllowed(true);

        // create the frame on the event dispatch thread
        if ((args != null) && (args.length > 0) && (args[0].length() != 0)) {
            EventQueue.invokeLater(new Runnable() {
                    public void run() {
                        File argFile = new File(args[0]);

                        //System.out.println("F " + argFile.getAbsolutePath());
                        //System.out.println("A " + argFile.isAbsolute());
                        // HS July 2008: check if the argument (filepath to eaf) is a relative
                        // path. If so let the jvm resolve it relative to the current directory
                        // (where ELAN is launched from)
                        if (!argFile.isAbsolute()) {
                            //System.out.println("F " + argFile.getAbsolutePath());
                            FrameManager.getInstance().createFrame(argFile.getAbsolutePath());
                        } else {
                            FrameManager.getInstance().createFrame(args[0]);
                        }
                    }
                });
        } else {
            EventQueue.invokeLater(new Runnable() {
                    public void run() {
                        FrameManager.getInstance().createEmptyFrame();
                    }
                });
        }

        // external launcher, currently only accepts imdi files to open an eaf
        // from another application/VM
        mpi.eudico.client.annotator.integration.ExternalLauncher.start();
    }

    /**
     * Returns the current version information as a string.
     *
     * @return the current version
     */
    public static String getVersionString() {
        return major + "." + minor + "." + micro;
    }
}
