/*
 * File:     ExportMediaCommand.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.Selection;
import mpi.eudico.client.annotator.ViewerManager2;

import mpi.eudico.client.annotator.export.ExportSmilDialog;

import mpi.eudico.client.annotator.player.ElanMediaPlayer;

import mpi.eudico.server.corpora.clom.Transcription;

import mpi.eudico.server.corpora.clomimpl.abstr.MediaDescriptor;
import mpi.eudico.server.corpora.clomimpl.dobes.EAF21TranscriptionStore;

import java.io.File;
import java.io.IOException;

import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;


/**
 *
 */
public class ExportMediaCommand implements Command {
    private String commandName;

    /**
     * Creates a new ExportSmilCommand instance
     *
     * @param theName DOCUMENT ME!
     */
    public ExportMediaCommand(String theName) {
        commandName = theName;
    }

    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are
     * correct.
     *
     * For the time being only the master media is exported
     * M2-edeit-cl only supports mpeg1 and mpeg2 files,
     * for wav clips use java code based on wav reader code
     *
     * @param receiver null
     * @param arguments the arguments:  <ul><li>arg[0] = the Viewer Manager
     *        (ViewerManager2)</li> </ul>
     */
    public void execute(Object receiver, Object[] arguments) {
        ViewerManager2 viewerManager = (ViewerManager2) arguments[0];

        if (viewerManager != null) {
            Selection selection = viewerManager.getSelection();

            if (selection != null) {
                // do something to point to source media and target media file name
                // and maybe also give the option to define a time interval independent of the current selection
                // for the time being the clip is saved in the fiel pointed to by a file chooser
                String saveDir = (String) Preferences.get("MediaDir", null);

                if (saveDir == null) {
                    saveDir = System.getProperty("user.dir");
                }

                JFileChooser chooser = new JFileChooser(saveDir);
                int option = chooser.showSaveDialog(ELANCommandFactory.getRootFrame(
                            viewerManager.getTranscription()));

                if (option == JFileChooser.APPROVE_OPTION) {
                    try {
                        String saveFileName = chooser.getSelectedFile()
                                                     .getAbsolutePath();
                        String curDir = new File(saveFileName).getParent();

                        if (curDir != null) {
                            Preferences.set("MediaDir", curDir, null);
                        }

                        // check for extension
                        if (!saveFileName.toLowerCase().endsWith(".mpg") &&
                                !saveFileName.toLowerCase().endsWith(".mpeg")) {
                            saveFileName += ".mpg";
                        }

                        new ClipThread(viewerManager, saveFileName).start();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Returns the name
     *
     * @return the name
     */
    public String getName() {
        return commandName;
    }

    /**
     * Start clipping by M2-edit in a separate thread. Waiting for the process to end
     * might halt the application.
     */
    private class ClipThread extends Thread {
        /** Holds value of property DOCUMENT ME! */
        ViewerManager2 viewerManager;

        /** Holds value of property DOCUMENT ME! */
        String saveFileName;

        /**
         * Creates a new ClipThread instance
         *
         * @param viewerManager DOCUMENT ME!
         * @param saveFileName DOCUMENT ME!
         */
        public ClipThread(ViewerManager2 viewerManager, String saveFileName) {
            super();
            this.viewerManager = viewerManager;
            this.saveFileName = saveFileName;
        }

        /**
         * DOCUMENT ME!
         */
        public void run() {
            if ((viewerManager == null) || (saveFileName == null)) {
                return;
            }

            ElanMediaPlayer player = viewerManager.getMasterMediaPlayer();
            long offset = player.getOffset();
            Selection selection = viewerManager.getSelection();
            long frameDuration = player.getMilliSecondsPerSample();
            long beginFrame = (offset + selection.getBeginTime()) / frameDuration;
            long endFrame = (long) Math.ceil(((double) offset +
                    selection.getEndTime()) / frameDuration);

            // M2-edit-class can handle unix type path delimiters, but when the file is on 
            // a network share it should start with "\\" instead of "//"
            String sourceFileName = player.getMediaDescriptor().mediaURL.substring(5);

            if (sourceFileName.startsWith("///")) {
                sourceFileName = sourceFileName.substring(3);
            } else {
                // network share, starting with //share/folder...
                sourceFileName.replace('/', '\\');
            }

            String[] command = new String[5];
            command[0] = "M2-edit-cl";
            command[1] = "/in:" + beginFrame;
            command[2] = "/out:" + endFrame;
            command[3] = sourceFileName;
            command[4] = saveFileName;
            System.out.println(command[0] + " " + command[1] + " " +
                command[2] + " " + command[3] + " " + command[4]);

            try {
                Process pr = Runtime.getRuntime().exec(command);
                int exit = pr.waitFor();

                if (exit != 0) {
                    System.out.println("Clipping exited abnormally: " + exit);
                }

                System.out.println("exported clip:" + saveFileName);
            } catch (InterruptedException ie) {
            } catch (IOException ioe) {
            }

            // apr 2008 HS: also export other videos
            List slavePlayers = viewerManager.getSlaveMediaPlayers();
            MediaDescriptor md;
            ElanMediaPlayer pl;

            for (int i = 0; i < slavePlayers.size(); i++) {
                pl = (ElanMediaPlayer) slavePlayers.get(i);
                md = pl.getMediaDescriptor();
                offset = pl.getOffset();

                if (md == null) {
                    continue;
                }

                System.out.println("Player: " + md.mediaURL);

                String sourceName = md.mediaURL.substring(5);

                if (sourceName.startsWith("///")) {
                    sourceName = sourceName.substring(3);
                } else {
                    // network share, starting with //share/folder...
                    sourceName.replace('/', '\\');
                }

                int stopIndex = saveFileName.lastIndexOf('.');
                String saveName = saveFileName.substring(0, stopIndex) + "_" +
                    (i) + saveFileName.substring(stopIndex);

                long frameDur = player.getMilliSecondsPerSample();
                long beginFr = (offset + selection.getBeginTime()) / frameDur;
                long endFr = (long) Math.ceil(((double) offset +
                        selection.getEndTime()) / frameDur);

                String[] comm = new String[5];
                comm[0] = "M2-edit-cl";
                comm[1] = "/in:" + beginFr;
                comm[2] = "/out:" + endFr;
                comm[3] = sourceName;
                comm[4] = saveName;

                //System.out.println(comm[0] + " " + comm[1] + " " + comm[2] + " " + comm[3] + " " + comm[4]);
                try {
                    Process pr = Runtime.getRuntime().exec(comm);

                    int exit = pr.waitFor();

                    if (exit != 0) {
                        System.out.println("Clipping exited abnormally: " +
                            exit);
                    }

                    System.out.println("exported clip:" + saveName);
                } catch (InterruptedException ie) {
                    System.out.println("export interrupted: " +
                        ie.getMessage());
                } catch (IOException ioe) {
                    System.out.println("export failed: " + ioe.getMessage());
                }
            }
        }
    }
}
