/*
 * File:     DCRCacheReaderWriter.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.dcr;

import mpi.dcr.DCSmall;

import mpi.eudico.util.IoUtil;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderAdapter;
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 * A reader and writer of the locally stored cache. In this first version the
 * GMT format is used.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class DCRCacheReaderWriter {
    /** Holds value of property DOCUMENT ME! */
    private final String struct = "struct";

    /** Holds value of property DOCUMENT ME! */
    private final String feat = "feat";

    /** Holds value of property DOCUMENT ME! */
    private final String dcr = "DCR"; //or DCS

    /** Holds value of property DOCUMENT ME! */
    private final String dcs = "DCS";

    /** Holds value of property DOCUMENT ME! */
    private final String dc = "DC";

    /** Holds value of property DOCUMENT ME! */
    private final String ai = "AI";

    /** Holds value of property DOCUMENT ME! */
    private final String ar = "AR";

    /** Holds value of property DOCUMENT ME! */
    private final String desc = "Desc";

    /** Holds value of property DOCUMENT ME! */
    private final String def = "definition";

    /** Holds value of property DOCUMENT ME! */
    private final String ident = "identifier";

    /** Holds value of property DOCUMENT ME! */
    private final String prof = "profile";

    /** Holds value of property DOCUMENT ME! */
    private final String broad = "broaderConceptGeneric";

    /** Holds value of property DOCUMENT ME! */
    private final String type = "type";

    /** Holds value of property DOCUMENT ME! */
    private final String id = "id";
    private DocumentBuilder db;
    private XMLReader reader;
    private XMLReaderAdapter xmlAdapter;
    private String filePath;

    /**
     * Creates a new DCRCacheReaderWriter instance
     *
     * @throws ParserConfigurationException parser config exception
     */
    public DCRCacheReaderWriter() throws ParserConfigurationException {
        super();

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        db = dbf.newDocumentBuilder();

        try {
            reader = XMLReaderFactory.createXMLReader(
                    "org.apache.xerces.parsers.SAXParser");
            reader.setFeature("http://xml.org/sax/features/namespaces", true);
            reader.setFeature("http://xml.org/sax/features/validation", true);
            reader.setFeature("http://apache.org/xml/features/validation/schema",
                false);
            reader.setFeature("http://apache.org/xml/features/validation/dynamic",
                false);
        } catch (SAXException se) {
            // log
            se.printStackTrace();
        }
         /*catch (IOException ioe) {
         ioe.printStackTrace();
         }*/
    }

    /**
     * Returns the path to the local file.
     *
     * @return Returns the filePath.
     */
    public String getFilePath() {
        return filePath;
    }

    /**
     * Sets the path to the cache file
     *
     * @param filePath The filePath to set.
     */
    public void setFilePath(String filePath) {
        this.filePath = filePath;
    }

    /**
     * Creates a GMT format DOM with DCR or DCS as root struct element.
     *
     * @param categories list of DCSmall objects
     *
     * @throws IOException if there is no filepath specified
     * @throws ParserConfigurationException if no Document could be created
     * @throws NullPointerException if the category list is null
     */
    public synchronized void save(List categories)
        throws IOException, ParserConfigurationException {
        if (filePath == null) {
            throw new IOException("No filepath specified.");
        }

        if (categories == null) {
            throw new NullPointerException("The list of categories is null");
        }

        if (db != null) {
            Document doc = db.newDocument();
            Element dcsElem = doc.createElement(struct);
            doc.appendChild(dcsElem);
            dcsElem.setAttribute(type, dcs);

            DCSmall dcsmall = null;

            for (int i = 0; i < categories.size(); i++) {
                dcsmall = (DCSmall) categories.get(i);

                Element dcEl = doc.createElement(struct);
                dcEl.setAttribute(type, dc);
                dcEl.setAttribute(id, dcsmall.getId());
                dcsElem.appendChild(dcEl);

                if ((dcsmall.getIdentifier() != null) &&
                        (dcsmall.getIdentifier().length() > 0)) {
                    Element aiEl = doc.createElement(struct);
                    aiEl.setAttribute(type, ai);
                    dcEl.appendChild(aiEl);

                    Element arEl = doc.createElement(struct);
                    arEl.setAttribute(type, ar);
                    aiEl.appendChild(arEl);

                    Element idEl = doc.createElement(feat);
                    idEl.setAttribute(type, ident);
                    idEl.appendChild(doc.createTextNode(dcsmall.getIdentifier()));
                    arEl.appendChild(idEl);
                }

                Element descEl = doc.createElement(struct);
                descEl.setAttribute(type, desc);
                dcEl.appendChild(descEl);

                if ((dcsmall.getDesc() != null) &&
                        (dcsmall.getDesc().length() > 0)) {
                    Element defEl = doc.createElement(feat);
                    defEl.setAttribute(type, def);
                    defEl.appendChild(doc.createTextNode(dcsmall.getDesc()));
                    descEl.appendChild(defEl);
                }

                if ((dcsmall.getProfiles() != null) &&
                        (dcsmall.getProfiles().length > 0)) {
                    for (int j = 0; j < dcsmall.getProfiles().length; j++) {
                        if (dcsmall.getProfiles()[j].length() == 0) {
                            continue;
                        }

                        Element prEl = doc.createElement(feat);
                        prEl.setAttribute(type, prof);
                        prEl.appendChild(doc.createTextNode(
                                dcsmall.getProfiles()[j]));
                        descEl.appendChild(prEl);
                    }
                }

                if ((dcsmall.getBroaderDCId() != null) &&
                        (dcsmall.getBroaderDCId().length() > 0)) {
                    Element brEl = doc.createElement(feat);
                    brEl.setAttribute(type, broad);
                    brEl.appendChild(doc.createTextNode(
                            dcsmall.getBroaderDCId()));
                    descEl.appendChild(brEl);
                }
            }

            // write
            try {
                IoUtil.writeEncodedFile("UTF-8", filePath,
                    doc.getDocumentElement());
            } catch (Exception ioe) {
                throw new IOException(ioe.getMessage());
            }
        }
    }

    /**
     * Reads the data categories from the cache.
     *
     * @return the list of categories from the cache
     *
     * @throws IOException any io exception
     */
    public synchronized List read() throws IOException {
        if (reader != null) {
            try {
                DcrGmtAdapter adapter = new DcrGmtAdapter();
                reader.setContentHandler(adapter);
                reader.parse(filePath);

                return adapter.getDCS();
            } catch (SAXException se) {
                // log
                // return null;//??
                se.printStackTrace();
            }
        }

        List datcats = new ArrayList();

        return datcats;
    }

    /**
     * A parser adapter for parsing GMT style dc selection file.
     *
     * @author Han Sloetjes
     */
    private class DcrGmtAdapter extends DefaultHandler {
        private List datcats;

        // parse time objects
        private String idAttr;
        private String identifierAttr;
        private String descAttr;
        private DCSmall dcsmall;
        private String broaderDCIdAttr;
        private List profiles;
        private String curStruct;
        private String curFeat;
        private String content = "";
        private int structLevel = 0;

        /** Holds value of property DOCUMENT ME! */
        boolean in = false;

        /**
         * Creates a new DcrGmtAdapter instance
         */
        public DcrGmtAdapter() {
            super();

            //super(xmlReader);
            datcats = new ArrayList();
            profiles = new ArrayList();
        }

        /**
         * Return the list of small Data Category objects.
         *
         * @return the list of data categories
         */
        public List getDCS() {
            return datcats;
        }

        /**
         * @see org.xml.sax.helpers.XMLReaderAdapter#characters(char[], int,
         *      int)
         */
        public void characters(char[] ch, int start, int length)
            throws SAXException {
            //System.out.println("characters..." + start + " " + length);
            // for some reason all kinds of in between element characters (spaces, newlines) are passed
            // to this method ??
            if (in) {
                content = new String(ch, start, length);

                if (curFeat == ident) {
                    identifierAttr = content;
                } else if (curFeat == def) {
                    descAttr = content;
                } else if (curFeat == broad) {
                    broaderDCIdAttr = content;
                } else if (curFeat == prof) {
                    profiles.add(content);
                }
            }
        }

        /**
         * @see org.xml.sax.helpers.XMLReaderAdapter#endElement(java.lang.String,
         *      java.lang.String, java.lang.String)
         */
        public void endElement(String uri, String localName, String qName)
            throws SAXException {
            //System.out.println("end..." + localName);
            in = false;

            if (localName.equals(struct)) {
                structLevel--;
            }

            if (structLevel == 1) { // dc level, dcs is 0

                if ((idAttr != null) && (profiles.size() > 0)) {
                    dcsmall = new DCSmall(null, idAttr, identifierAttr);
                    dcsmall.setBroaderDCId(broaderDCIdAttr);
                    dcsmall.setProfiles((String[]) profiles.toArray(
                            new String[] {  }));
                    dcsmall.setDesc(descAttr);
                    dcsmall.setLoaded(true);
                    datcats.add(dcsmall);
                }
            }
        }

        /**
         * @see org.xml.sax.helpers.XMLReaderAdapter#parse(java.lang.String)
         */

        /*
           public void parse(String systemId) throws IOException, SAXException {
               super.parse(systemId);
           }
         */

        /**
         * @see org.xml.sax.helpers.XMLReaderAdapter#startElement(java.lang.String,
         *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
         */
        public void startElement(String uri, String localName, String qName,
            Attributes atts) throws SAXException {
            //System.out.println("start..." + localName);
            if (localName.equals(struct)) {
                structLevel++;

                String ty = atts.getValue(type);

                if (dc.equals(ty)) {
                    curStruct = dc;

                    // reset
                    idAttr = atts.getValue(id);
                    identifierAttr = null;
                    descAttr = null;
                    broaderDCIdAttr = null;
                    profiles.clear();
                    content = "";
                } else if (ar.equals(ty)) {
                    curStruct = ar;
                } else if (desc.equals(ty)) {
                    curStruct = desc;
                }
            } else if (localName.equals(feat)) {
                in = true;

                String fty = atts.getValue(type);

                if (ident.equals(fty)) {
                    curFeat = ident;
                } else if (def.equals(fty)) {
                    curFeat = def;
                } else if (prof.equals(fty)) {
                    curFeat = prof;
                } else if (broad.equals(fty)) {
                    curFeat = broad;
                }
            }
        }
    }
}
