package mpi.eudico.client.annotator.md.imdi;

import mpi.eudico.client.annotator.util.ClientLogger;

import mpi.eudico.client.util.MutableInt;

import mpi.eudico.util.DocumentNotLoadedException;

import mpi.util.OurURL;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.File;
import java.io.IOException;

import java.net.MalformedURLException;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;


/**
 * Creates a document, DOM based, and offers methods to interact with it.
 * Intended for retrieval of metadata keys (elements) and their values. Read
 * only.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class ImdiDoc {
    private Document doc;
    private List<String> allKeys = new ArrayList<String>();
    private Map<String, String> mdValuesMap = new LinkedHashMap<String, String>();
    private final String KEYS_KEY = "Keys.Key";

    /**
     * Creates a new ImdiDoc instance
     *
     * @param filePath the path to an imdi file
     *
     * @throws DocumentNotLoadedException thrown when a document could not be
     *         created
     */
    public ImdiDoc(String filePath) throws DocumentNotLoadedException {
        super();

        OurURL ourl = null;

        try {
            File file = new File(filePath);
            ourl = new OurURL(file.toURL());
        } catch (MalformedURLException mue) {
            mue.printStackTrace();
        }

        DocumentBuilder docBuilder;
        //DocumentBuilderFactory factory = org.apache.xerces.jaxp.DocumentBuilderFactoryImpl.newInstance();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringElementContentWhitespace(true);
        factory.setValidating(false);
        factory.setNamespaceAware(true);

        try {
            String fileName = ourl.toString();
            String uri = fileName.substring(0, fileName.lastIndexOf('/') + 1);
            InputSource fileSource = new InputSource(ourl.openStream());
            fileSource.setSystemId(uri);
            // the following can throw a ParserConfigurationException:
            docBuilder = factory.newDocumentBuilder();
            factory.setValidating(true);
            factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage",
                "http://www.w3.org/2001/XMLSchema");
            //          factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource",
            //              "http://www.mpi.nl/IMDI/schemas/xsd/IMDI_3.0.xsd");
            // the openStream can throw an IOException:
            factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource",
                this.getClass()
                    .getResource("/mpi/imdi/api/resources/IMDI_3.0.xsd")
                    .openStream());
            // the following can throw a ParserConfigurationException:
            docBuilder = factory.newDocumentBuilder();

            doc = docBuilder.parse(fileSource);

            //doc.normalize();//??
            //doc.normalizeDocument();//??
        } catch (SAXParseException spe) {
            //spe.printStackTrace();
            ClientLogger.LOG.warning("Cannot parse file: " + spe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                spe);
        } catch (SAXException sxe) {
            //sxe.printStackTrace();
            ClientLogger.LOG.warning("Cannot parse file: " + sxe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                sxe);
        } catch (IllegalArgumentException iae) {
            //iae.printStackTrace();
            ClientLogger.LOG.warning(
                "Cannot set attributes for document builder: " +
                iae.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                iae);
        } catch (ParserConfigurationException pce) {
            //pce.printStackTrace();
            ClientLogger.LOG.warning("Cannot set configure the parser: " +
                pce.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                pce);
        } catch (IOException ioe) {
            //ioe.printStackTrace();
            ClientLogger.LOG.warning("Cannot read the Imdi file: " +
                ioe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                ioe);
        }

        // create fully qualified id's for each element
        createIds(doc, doc.getDocumentElement(), null);

        //printChildren(doc);
    }

    /**
     * Creates a new ImdiDoc instance from an InputSource
     *
     * @param inputSource the input source
     *
     * @throws DocumentNotLoadedException thrown when a document could not be
     *         created
     */
    public ImdiDoc(InputSource inputSource) throws DocumentNotLoadedException {
        super();

        DocumentBuilder docBuilder;
        //DocumentBuilderFactory factory = org.apache.xerces.jaxp.DocumentBuilderFactoryImpl.newInstance();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringElementContentWhitespace(true);
        factory.setValidating(false);
        factory.setNamespaceAware(true);

        try {
            //String uri = fileName.substring(0, fileName.lastIndexOf('/') + 1);
            //inputSource.setSystemId(uri);
            // the following can throw a ParserConfigurationException:
            docBuilder = factory.newDocumentBuilder();
            factory.setValidating(true);
            factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage",
                "http://www.w3.org/2001/XMLSchema");
            //          factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource",
            //              "http://www.mpi.nl/IMDI/schemas/xsd/IMDI_3.0.xsd");
            // the openStream can throw an IOException:
            factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource",
                this.getClass()
                    .getResource("/mpi/imdi/api/resources/IMDI_3.0.xsd")
                    .openStream());
            // the following can throw a ParserConfigurationException:
            docBuilder = factory.newDocumentBuilder();

            doc = docBuilder.parse(inputSource);

            //doc.normalize();//??
            //doc.normalizeDocument();//??
        } catch (SAXParseException spe) {
            //spe.printStackTrace();
            ClientLogger.LOG.warning("Cannot parse file: " + spe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                spe);
        } catch (SAXException sxe) {
            //sxe.printStackTrace();
            ClientLogger.LOG.warning("Cannot parse file: " + sxe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                sxe);
        } catch (IllegalArgumentException iae) {
            //iae.printStackTrace();
            ClientLogger.LOG.warning(
                "Cannot set attributes for document builder: " +
                iae.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                iae);
        } catch (ParserConfigurationException pce) {
            //pce.printStackTrace();
            ClientLogger.LOG.warning("Cannot set configure the parser: " +
                pce.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                pce);
        } catch (IOException ioe) {
            //ioe.printStackTrace();
            ClientLogger.LOG.warning("Cannot read the Imdi file: " +
                ioe.getMessage());
            throw new DocumentNotLoadedException("Could not construct Imdi DOM: ",
                ioe);
        }

        // create fully qualified id's for each element
        createIds(doc, doc.getDocumentElement(), null);

        //printChildren(doc);
    }
    
    /**
     * Creates (temporary) id's for all elements. The id's are the "fully
     * qualified" element paths, that can be used as the keys in key-value
     * pairs.  Fills the "all keys" list.
     *
     * @param doc the document
     * @param elm the element
     * @param parentId the id
     */
    private void createIds(Document doc, Element elm, String parentId) {
        if (parentId != null) {
            elm.setAttribute("id", parentId + "." + elm.getNodeName());
        } else {
            elm.setAttribute("id", elm.getNodeName());
        }

        //elm.setIdAttribute("id", true); // only useful if getElementById is exploited

        String nextId = elm.getAttribute("id");

        //System.out.println("ID: " + nextId);
        if (!allKeys.contains(nextId)) {
            allKeys.add(nextId);
        }

        allKeys.remove(ImdiConstants.METATRANSCRIPT);

        NodeList childList = elm.getChildNodes();
        Node nd;

        for (int i = 0; i < childList.getLength(); i++) {
            nd = childList.item(i);

            if (nd instanceof Element) {
                createIds(doc, (Element) nd, nextId);
            }
        }

        /* test
           Element actors = doc.getElementById(
                   "METATRANSCRIPT.Session.MDGroup.Actors.Actor.Name");
           if (actors != null) {
               System.out.println("Actor: " + actors.getNodeName() + " " +
                   actors.getTextContent());
           } else {
               System.out.println("Element not found...");
           }
         */
    }

    /**
     * Returns a List containing the metadata keys.
     *
     * @return the keys
     */
    public List getKeys() {
        return allKeys;
    }

    /**
     * Returns the values for the specified keys; a single key can have
     * multiple values  (or a key can occur more than once, e.g.
     * Actors.Actor.Name).
     *
     * @param key the key
     *
     * @return the list of values.
     */
    public List getValues(String key) {
        if (key == null) {
            return null;
        }

        // Could also start by finding all elements with a certain tag name and 
        // filter the right ones out through the parent nodes
        /*
           if (key.indexOf(".") > -1) {
               int li = key.lastIndexOf('.');
               String elName = key.substring(li + 1);
               NodeList nl = doc.getElementsByTagName(elName);
               for (int i = 0; i < nl.getLength(); i++) {
                   Node nd = nl.item(i);
                   if (nd instanceof Element) {
                       System.out.println("El id: " + ((Element)nd).getAttribute("id"));
                   }
               }
           }
         */
        List<Element> elems = getElementsById(key);

        if (elems.size() > 0) {
            List<String> vals = new ArrayList<String>(elems.size());
            Element elem;

            for (int i = 0; i < elems.size(); i++) {
                elem = elems.get(i);

                if ((elem.getFirstChild() != null) &&
                        (elem.getFirstChild().getNodeType() == Node.TEXT_NODE)) {
                    vals.add(elem.getFirstChild().getNodeValue());
                } else {
                    vals.add("");
                }
            }

            return vals;
        }

        return null;
    }

    /**
     * Returns the value for a certain key.
     *
     * @param key the key
     *
     * @return the value or null
     */
    public String getValue(String key) {
        if (mdValuesMap != null) {
            return mdValuesMap.get(key);
        }

        return null;
    }

    /**
     * Returns a Map with all key-value pairs given the list of keys.
     *
     * @param keys the keys
     *
     * @return a map containing all key value pairs
     */
    public Map<String, String> getValuesForKeys(List<String> keys, boolean includeChildren) {
        if ((keys == null) || (keys.size() == 0)) {
            return null;
        }

        Map<String, String> mdValues = new LinkedHashMap<String, String>();

        addKeyAndValue(keys, doc, mdValues, new MutableInt(0), includeChildren);

        return mdValues;
    }

    /**
     * There can be multiple elements with the same id (path)!
     *
     * @param key the key
     *
     * @return a list containing all occurrences of the particular element
     */
    private List<Element> getElementsById(String key) {
        if (key != null) {
            List<Element> elems = new ArrayList<Element>(5);
            addChildElementsById(key, doc, elems);

            return elems;
        }

        return null;
    }

    /**
     * The id's are the path to the elements and there can be duplicates!
     *
     * @param key the (fully qualified) key
     * @param node the parent node
     * @param elemList the list to add to
     */
    private void addChildElementsById(String key, Node node,
        List<Element> elemList) {
        if (node != null) {
            NodeList nodes = node.getChildNodes();
            Node nd;
            Element el;

            for (int i = 0; i < nodes.getLength(); i++) {
                nd = nodes.item(i);

                if (nd instanceof Element) {
                    el = (Element) nd;

                    if (key.equals(el.getAttribute("id"))) {
                        elemList.add(el);
                    }
                }

                addChildElementsById(key, nd, elemList);
            }
        }
    }

    /**
     * Recursively checks the Node's children for an id that is in the list of
     * keys. Id's can occur multiple times since some elements can occur more
     * than once.
     *
     * @param keys the keys to search for
     * @param node the node to start with
     * @param mdValues the map to add valid key-value pairs to
     * @param index a counter to prepend to the key
     * @param includeChildren if true all child elements are also added (regardless of whether 
     * their id is in the list of keys
     */
    private void addKeyAndValue(List<String> keys, Node node,
        Map<String, String> mdValues, MutableInt index, boolean includeChildren) {
        if (node == null) {
            return;
        }

        NodeList nodes = node.getChildNodes();
        Node nd;
        Element elem;
        String key = null;
        String val = null;

        for (int i = 0; i < nodes.getLength(); i++) {
            nd = nodes.item(i);

            if (nd instanceof Element) {
                elem = (Element) nd;
                key = elem.getAttribute("id");

                if (keys.contains(key)) {
                    if ((elem.getFirstChild() != null) &&
                            (elem.getFirstChild().getNodeType() == Node.TEXT_NODE)) {
                        val = elem.getFirstChild().getNodeValue();
                    } else {
                        val = "";
                    }

                    // special treatment of Keys key-value pairs
                    if (key.endsWith(KEYS_KEY)) {
                        String name = elem.getAttribute("Name");

                        if (name != null) {
                            key += ("[" + name + "]");
                        }
                    }

                    // only add if the value is non empty??
                    mdValues.put(("(" + index.intValue + ")" + key), val);

                    if (includeChildren) {
                        Node n2 = null;
                        NodeList childEl = elem.getElementsByTagName("*"); // all child elements

                        if (childEl.getLength() > 0) {
                            for (int j = 0; j < childEl.getLength(); j++) {
                                n2 = childEl.item(j);

                                if (n2 instanceof Element) {
                                    index.intValue++;
                                    addChildKeyAndValue(childEl.item(j),
                                        mdValues, index, includeChildren);
                                }
                            }
                        }
                    }
                }
            }

            index.intValue++;
            addKeyAndValue(keys, nd, mdValues, index, includeChildren);
        }
    }

    /**
     * Adds all child elements of the specified node, recursively.
     *
     * @param node the parent node
     * @param mdValues the map to add the values to
     * @param index a counter to prepend to the key
     * @param includeChildren if true all child elements are also added (regardless of whether 
     * their id is in the list of keys
     */
    private void addChildKeyAndValue(Node node, Map<String, String> mdValues,
        MutableInt index, boolean includeChildren) {
        if (node == null) {
            return;
        }

        Element elem;
        String key = null;
        String val = null;

        if (node instanceof Element) {
            elem = (Element) node;
            key = elem.getAttribute("id");

            if ((elem.getFirstChild() != null) &&
                    (elem.getFirstChild().getNodeType() == Node.TEXT_NODE)) {
                val = elem.getFirstChild().getNodeValue();
            } else {
                val = "";
            }

            // special treatment of Keys key-value pairs
            if (key.endsWith(KEYS_KEY)) {
                String name = elem.getAttribute("Name");

                if (name != null) {
                    key += ("[" + name + "]");
                }
            }

            // only add if the value is non empty??
            mdValues.put(("(" + index.intValue + ")" + key), val);

            if (includeChildren) {
                Node n2 = null;
                NodeList childEl = elem.getElementsByTagName("*"); // all child elements

                if (childEl.getLength() > 0) {
                    for (int j = 0; j < childEl.getLength(); j++) {
                        n2 = childEl.item(j);

                        if (n2 instanceof Element) {
                            index.intValue++;
                            addChildKeyAndValue(childEl.item(j), mdValues, index, includeChildren);
                        }
                    }
                }
            }
        }
    }

    private void printChildren(Node node) {
        NodeList nodes = node.getChildNodes();
        Element el;

        for (int i = 0; i < nodes.getLength(); i++) {
            Node nd = nodes.item(i);
            System.out.println("N: " + nd.getNodeName() + " V: " +
                nd.getNodeValue());

            if (nd instanceof Element) {
                el = (Element) nd;
                System.out.println("Id: " + el.getAttribute("id"));

                if ((el.getFirstChild() != null) &&
                        (el.getFirstChild().getNodeType() == Node.TEXT_NODE)) {
                    System.out.println("Text: " +
                        el.getFirstChild().getNodeValue());
                }

                System.out.println("Num Ch.: " +
                    el.getChildNodes().getLength());
            }

            if (nd.hasChildNodes()) {
                printChildren(nd);
            }
        }
    }
}
