/*
 * File:     AnnotationDensityViewer.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.viewer;

import mpi.eudico.client.annotator.ElanLocale;

import mpi.eudico.client.mediacontrol.ControllerEvent;

import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;

import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;

import mpi.eudico.server.corpora.event.ACMEditEvent;
import mpi.eudico.server.corpora.event.ACMEditListener;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;

import java.util.List;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.MenuElement;
import javax.swing.SwingUtilities;


/**
 * A viewer that reflects the annotation density and distribution over
 * the media duration by short vertical lines.
 *
 * @author MP
 */
public class AnnotationDensityViewer extends AbstractViewer
    implements ACMEditListener, MouseListener, ActionListener {
    private static Dimension SLIDERDIMENSION = new Dimension(600, 30);

    /** Holds value of property DOCUMENT ME! */
    final private int imageHeight = 5;
    private int imageWidth;
    private Transcription transcription;
    private BufferedImage bi;
    private Graphics2D big2d;
    private BufferedImage displayImage;

    // tiers popup menu
    private JPopupMenu popup;
    private ButtonGroup tiersBG;
    private JRadioButtonMenuItem allTiersRB;
    private boolean showAllTiers = true;
    private String currentTierName = null;

    /**
     * Constructor
     *
     * @param transcription the transcription containing the annotations
     */
    public AnnotationDensityViewer(Transcription transcription) {
        super();

        this.transcription = transcription;

        setPreferredSize(SLIDERDIMENSION);

        paintBuffer();
    }

    /**
     * Returnss minimum size of the slider See also getPreferredSize
     *
     * @return the minimum size
     */
    public Dimension getMinimumSize() {
        return SLIDERDIMENSION;
    }

    /**
     * Returns minimum size of the slider See also getMinimumSize
     *
     * @return the preferred size
     */
    public Dimension getPreferredSize() {
        return SLIDERDIMENSION;
    }

    private void paintBuffer() {
        if (getWidth() > 0) {
            imageWidth = getWidth();
        } else {
            imageWidth = SLIDERDIMENSION.width;
        }

        if ((bi == null) || (bi.getWidth() != imageWidth) ||
                (bi.getHeight() != imageHeight)) {
            bi = new BufferedImage(imageWidth, imageHeight,
                    BufferedImage.TYPE_INT_RGB);
            big2d = bi.createGraphics();
        }

        //big2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
        //    RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        //paint the background color
        big2d.setColor(Color.lightGray);

        //complete rectangle
        big2d.fillRect(0, 0, imageWidth, imageHeight);
        big2d.setColor(Color.white);

        //upper line
        big2d.fillRect(0, 0, imageWidth, 1);

        //left line
        big2d.drawLine(0, 0, 0, imageHeight);
        big2d.setColor(Color.darkGray);

        //lower line
        big2d.fillRect(0, imageHeight - 1, imageWidth, 1);

        //paint the vertical line where the annotations are
        big2d.setColor(Color.darkGray.brighter());

        Vector vTiers = new Vector(0);
        Vector annotations = null;
        TierImpl tier;
        Annotation ann;
        int begintime;
        int endtime;
        int midtime;
        int intx;
        int duration = (int) getMediaDuration();

        if (duration == 0) {
            displayImage = bi;
            repaint();

            return;
        }

        vTiers = transcription.getTiers();

        int vTiers_size = vTiers.size();

        if (showAllTiers) {
            for (int i = 0; i < vTiers_size; i++) {
                tier = (TierImpl) vTiers.elementAt(i);

                // aug 2005: if the tier is of type SymbolicAssociation it is 
                // guaranteed that the line for each annotation has been or 
                // will be drawn for the parent annotation
                if ((tier.getLinguisticType().getConstraints() != null) &&
                        (tier.getLinguisticType().getConstraints()
                                 .getStereoType() == Constraint.SYMBOLIC_ASSOCIATION)) {
                    continue;
                }

                annotations = ((TierImpl) tier).getAnnotations();

                int size = annotations.size();

                for (int j = 0; j < size; j++) {
                    ann = (Annotation) annotations.elementAt(j);
                    begintime = (int) ann.getBeginTimeBoundary();
                    endtime = (int) ann.getEndTimeBoundary();

                    midtime = (begintime + endtime) / 2;

                    //intx = (int) ((imageWidth * midtime) / duration);
                    intx = (int) (imageWidth * ((float) midtime / duration));
                    big2d.drawLine(intx, 0, intx, imageHeight);
                }
            }
        } else if (currentTierName != null) {
            tier = (TierImpl) transcription.getTierWithId(currentTierName);

            if (tier != null) {
                annotations = tier.getAnnotations();

                int size = annotations.size();

                for (int j = 0; j < size; j++) {
                    ann = (Annotation) annotations.elementAt(j);
                    begintime = (int) ann.getBeginTimeBoundary();
                    endtime = (int) ann.getEndTimeBoundary();

                    midtime = (begintime + endtime) / 2;

                    //intx = (int) ((imageWidth * midtime) / duration);
                    intx = (int) (imageWidth * ((float) midtime / duration));
                    big2d.drawLine(intx, 0, intx, imageHeight);
                }
            }
        }

        //displayImage = bi.getSubimage(0, 0, imageWidth, imageHeight);
        displayImage = bi;
        repaint();
    }

    /**
     * Handles the painting of the slider Consists of a vertical bar, a
     * crosshair and maybe a selection
     *
     * @param g the graphics object
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        if (getWidth() != imageWidth) {
            paintBuffer();
        }

        Graphics2D g2d = (Graphics2D) g;

        if (displayImage != null) {
            g2d.drawImage(displayImage, 0, 15, this);
        }
    }

    /**
     * Creates a popup menu, with a radio button for each tier and one for
     * all tiers.
     */
    private void createPopupMenu() {
        popup = new JPopupMenu("Density");
        tiersBG = new ButtonGroup();
        allTiersRB = new JRadioButtonMenuItem(ElanLocale.getString(
                    "MultiTierControlPanel.Menu.ShowAllTiers"));
        allTiersRB.setSelected(true);
        allTiersRB.addActionListener(this);
        tiersBG.add(allTiersRB);
        popup.add(allTiersRB);
        popup.addSeparator();

        if (transcription != null) {
            List tiers = transcription.getTiers();
            int size = tiers.size();
            String name;
            JRadioButtonMenuItem trmi;

            for (int i = 0; i < size; i++) {
                name = ((TierImpl) tiers.get(i)).getName();
                trmi = new JRadioButtonMenuItem(name);
                trmi.setActionCommand(name);
                trmi.addActionListener(this);
                tiersBG.add(trmi);
                popup.add(trmi);
            }
        }
    }

    /**
     * Updates a popup menu item if a tier name has changed.
     *
     * @param t the tier
     */
    private void tierChanged(TierImpl t) {
        if (popup != null) {
            String name = t.getName();
            JRadioButtonMenuItem trmi = null;
            Object item;
            MenuElement[] items = popup.getSubElements();

            for (int i = 0; i < items.length; i++) {
                item = items[i];

                if (item instanceof JRadioButtonMenuItem) {
                    trmi = (JRadioButtonMenuItem) item;

                    if (trmi.getText().equals(name) && (trmi != allTiersRB)) {
                        trmi.setText(name);
                        trmi.setActionCommand(name);

                        if (!showAllTiers && name.equals(currentTierName)) {
                            currentTierName = name;
                        }

                        break;
                    }
                }
            }
        }

        paintBuffer();
    }

    /**
     * If there is already a popup menu, an item for the new tier is added.
     *
     * @param t the new tier
     */
    private void tierAdded(TierImpl t) {
        if (popup != null) {
            String name = t.getName();
            JRadioButtonMenuItem trmi = new JRadioButtonMenuItem(name);
            trmi.setActionCommand(name);
            trmi.addActionListener(this);
            tiersBG.add(trmi);
            popup.add(trmi);
        }
    }

    /**
     * If there is already a popup menu the item for the tier is removed.
     * If the removed tier was the current tier, all tiers are shown.
     *
     * @param t the removed tier
     */
    private void tierRemoved(TierImpl t) {
        if (popup != null) {
            String name = t.getName();
            JRadioButtonMenuItem trmi = null;
            Object item;
            MenuElement[] items = popup.getSubElements();

            for (int i = 0; i < items.length; i++) {
                item = items[i];

                if (item instanceof JRadioButtonMenuItem) {
                    trmi = (JRadioButtonMenuItem) item;

                    if (trmi.getText().equals(name) && (trmi != allTiersRB)) {
                        trmi.removeActionListener(this);
                        tiersBG.remove(trmi);
                        popup.remove(trmi);

                        if (!showAllTiers && name.equals(currentTierName)) {
                            // show all tiers
                            showAllTiers = true;
                            currentTierName = null;
                            allTiersRB.setSelected(true);
                        }

                        break;
                    }
                }
            }
        }

        paintBuffer();
    }

    /**
     * The ACM editing event handling. Updates the popup menu and repaints.
     *
     * @param e the event
     */
    public void ACMEdited(ACMEditEvent e) {
        switch (e.getOperation()) {
        case ACMEditEvent.ADD_TIER:

            if (e.getModification() instanceof TierImpl) {
                tierAdded((TierImpl) e.getModification());
            }

            break;

        case ACMEditEvent.REMOVE_TIER:

            if (e.getModification() instanceof TierImpl) {
                tierRemoved((TierImpl) e.getModification());
            }

            break;

        case ACMEditEvent.CHANGE_TIER:

            if (e.getInvalidatedObject() instanceof TierImpl) {
                tierChanged((TierImpl) e.getInvalidatedObject());
            }

            break;

        case ACMEditEvent.ADD_ANNOTATION_HERE:
        case ACMEditEvent.ADD_ANNOTATION_BEFORE:
        case ACMEditEvent.ADD_ANNOTATION_AFTER:
        case ACMEditEvent.REMOVE_ANNOTATION:
        case ACMEditEvent.CHANGE_ANNOTATION_TIME:
        case ACMEditEvent.CHANGE_ANNOTATIONS:
            paintBuffer();
        }
    }

    /**
     * Updates the all tiers menu item, if it exists.
     */
    public void updateLocale() {
        if (allTiersRB != null) {
            allTiersRB.setText(ElanLocale.getString(
                    "MultiTierControlPanel.Menu.ShowAllTiers"));
        }
    }

    /**
     * Ignored
     */
    public void updateActiveAnnotation() {
    }

    /**
     * Ignored
     */
    public void updateSelection() {
    }

    /**
     * Ignored
     *
     * @param event the event
     */
    public void controllerUpdate(ControllerEvent event) {
    }

    /**
     * DOCUMENT ME!
     */
    public void preferencesChanged() {
        // stub
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void mouseClicked(MouseEvent e) {
        // TODO Auto-generated method stub
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * SHows the popup menu.
     */
    public void mousePressed(MouseEvent e) {
        if ((SwingUtilities.isRightMouseButton(e) &&
                ((e.getButton() == MouseEvent.BUTTON1) ^ e.isMetaDown())) ||
                e.isPopupTrigger()) {
            if (popup == null) {
                createPopupMenu();
            }

            popup.show(this, e.getPoint().x, e.getPoint().y);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Changes the selected tier.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == allTiersRB) {
            showAllTiers = true;
            currentTierName = null;
        } else {
            showAllTiers = false;
            currentTierName = e.getActionCommand();
        }

        paintBuffer();
    }
}


//end of AnnotationDensityViewer
